from django.shortcuts import render, redirect
from django.views.generic import ListView, CreateView, TemplateView, DetailView, UpdateView, View, DeleteView
from django.views.generic.base import RedirectView
from django.core.urlresolvers import reverse_lazy
from django.conf import settings
from datetime import datetime

from incidents.models import *
from incidents.forms import *

from sender.send import Sender

from braces.views import LoginRequiredMixin

# Main dashboard
class DashboardView(LoginRequiredMixin, TemplateView):
    template_name = "dashboard.html"

#
# This part handles actual incidents either received from other providers or
# created by internal human incident handlers
#
class IncidentList(LoginRequiredMixin, ListView):
    model = Incident
    template_name = "incidents/list.html"
    context_object_name = "items"
    paginate_by = 10

class IncidentCreate(LoginRequiredMixin, CreateView):
    form_class = IncidentForm
    model = Incident
    template_name = "incidents/edit.html"
    success_url = reverse_lazy('incident-list')
    
    def get_context_data(self, **kwargs):
        context = super(IncidentCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context
    
    def get_initial(self):
        initial = super(IncidentCreate, self).get_initial()
        
        parent = self.request.GET.get('parent')
        initial['parent'] = parent
        
        if parent:
            incident = Incident.objects.get(pk = parent)
            initial['occurrence_time'] = incident.occurrence_time
            initial['detection_time'] = incident.detection_time
        
        return initial
    
    def form_valid(self, form):
        result = super(IncidentCreate, self).form_valid(form)
        
        custom_fields = CustomField.objects.filter(incident_type = self.object.type)
        
        for field in custom_fields:
            value = CustomFieldValue()
            value.type = field
            value.incident = self.object
            value.value = self.request.POST.get(str(field.id), "")
            
            value.save()
        
        status, created = IncidentNotificationStatus.objects.get_or_create(incident = self.object)
        status.notified = False
        status.save()
        
        return result

class IncidentDetails(LoginRequiredMixin, DetailView):
    model = Incident
    template_name = "incidents/details.html"
    
    def get_context_data(self, **kwargs):
        context = super(IncidentDetails, self).get_context_data(**kwargs)
        pk = self.kwargs.get('pk')
        context['attachments'] = Attachment.objects.filter(incident__id = pk)
        context['custom_values'] = CustomFieldValue.objects.filter(incident__id = pk)
        
        return context

class IncidentEdit(LoginRequiredMixin, UpdateView):
    model = Incident
    form_class = IncidentForm
    template_name = "incidents/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-details', kwargs={'pk': self.kwargs.get('pk')})
    
    def get_context_data(self, **kwargs):
        context = super(IncidentEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        values = CustomFieldValue.objects.filter(incident__id = self.kwargs.get('pk'))
        dictionary = {}
        for value in values:
            dictionary[str(value.type.id)] = value.value
            
        context['custom_values'] = dictionary
        context['incidentId'] = self.object.id
        
        return context
    
    def form_valid(self, form):
        result = super(IncidentEdit, self).form_valid(form)
        
        
        custom_fields = CustomField.objects.filter(incident_type = self.object.type)
        
        for field in custom_fields:
            value = CustomFieldValue()
            value.type = field
            value.incident = self.object
            value.value = self.request.POST.get(str(field.id), "")
            
            value.save()
        
        status, created = IncidentNotificationStatus.objects.get_or_create(incident = self.object)
        status.notified = False
        status.save()

        return result

#
# Subscribers are the next link in the incident exchange chain. A subscriber could
# be a local authority, a customer, a log system, or another provider.
#
class SubscriberList(LoginRequiredMixin, ListView):
    model = Subscriber
    context_object_name = "items"
    paginate_by = 10
    template_name = "subscribers/list.html"

class SubscriberCreate(LoginRequiredMixin, CreateView):
    form_class = SubscriberForm
    model = Subscriber
    template_name = "subscribers/edit.html"
    success_url = reverse_lazy('subscriber-list')
    
    def form_valid(self, form):
        result = super(SubscriberCreate, self).form_valid(form)
        
        # TODO add auth
        
        
        return result
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class SubscriberEdit(LoginRequiredMixin, UpdateView):
    model = Subscriber
    form_class = SubscriberForm
    template_name = "subscribers/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-details', kwargs={'pk': self.kwargs.get('pk')})
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class SubscriberDetails(LoginRequiredMixin, DetailView):
    model = Subscriber
    template_name = "subscribers/details.html"
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberDetails, self).get_context_data(**kwargs)
        
        subscriber = Subscriber()
        subscriber.id = self.kwargs.get('pk')
        
        context['subscriptions'] = NotificationType.objects.filter(subscriber = subscriber)
        
        return context

class SubscriberDelete(LoginRequiredMixin, DeleteView):
    model = Subscriber
    success_url = reverse_lazy('subscriber-list')
    template_name = "subscribers/delete.html"

#
# Providers are the previous link in the incident exchange chain. This is where the incident system
# receives incidents from. That be sensors, service providers, incident pools, etc.
#
class ProviderList(LoginRequiredMixin, ListView):
    model = Provider
    context_object_name = "items"
    paginate_by = 10
    template_name = "providers/list.html"

class ProviderCreate(LoginRequiredMixin, CreateView):
    form_class = ProviderForm
    model = Provider
    template_name = "providers/edit.html"
    success_url = reverse_lazy('provider-list')
    
    def form_valid(self, form):
        result = super(ProviderCreate, self).form_valid(form)
        
        # TODO add auth
        
        return result
    
    def get_context_data(self, **kwargs):
        context = super(ProviderCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class ProviderDetails(LoginRequiredMixin, DetailView):
    model = Provider
    template_name = "providers/details.html"
    
    def get_context_data(self, **kwargs):
        context = super(ProviderDetails, self).get_context_data(**kwargs)
        
        return context

class ProviderEdit(LoginRequiredMixin, UpdateView):
    model = Provider
    form_class = ProviderForm
    template_name = "providers/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('provider-details', kwargs={'pk': self.kwargs.get('pk')})
    
    def get_context_data(self, **kwargs):
        context = super(ProviderEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class ProviderDelete(LoginRequiredMixin, DeleteView):
    model = Provider
    success_url = reverse_lazy('provider-list')
    template_name = "providers/delete.html"

#
# Incident types is a way of dynamically categorize incidents and control which 
# incidents each subscriber is allowed to receive.
#
class IncidentTypeList(LoginRequiredMixin, ListView):
    model = IncidentType
    context_object_name = "items"
    paginate_by = 10
    template_name = "incidents/types/list.html"
    
    def get_queryset(self):
        return self.model.objects.filter(provider = getattr(settings, 'SENDER'))

class IncidentTypeCreate(LoginRequiredMixin, CreateView):
    model = IncidentType
    form_class = IncidentTypeForm
    template_name = "incidents/types/edit.html"
    success_url = reverse_lazy('incident-type-list')
    
    def get_context_data(self, **kwargs):
        context = super(IncidentTypeCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class IncidentTypeDetails(LoginRequiredMixin, DetailView):
    model = IncidentType
    template_name = "incidents/types/details.html"
    
    def get_context_data(self, **kwargs):
        context = super(IncidentTypeDetails, self).get_context_data(**kwargs)
        
        context['triggers'] = TriggerType.objects.filter(incident_type = self.object)
        context['custom_fields'] = CustomField.objects.filter(incident_type = self.object)
        
        return context

class IncidentTypeEdit(LoginRequiredMixin, UpdateView):
    model = IncidentType
    form_class = IncidentTypeForm
    template_name = "incidents/types/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('pk')})
    
    def get_context_data(self, **kwargs):
        context = super(IncidentTypeEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class IncidentTypeDelete(LoginRequiredMixin, DeleteView):
    model = IncidentType
    template_name = "incidents/types/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('pk')})

#
# Triggers Types are a way of defining which conditions the subscriber might select to demand
# a notification 
#
class TriggerTypeEdit(LoginRequiredMixin, UpdateView):
    model = TriggerType
    template_name = "triggers/types/edit.html"
    form_class = TriggerTypeForm
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})
    
    def get_context_data(self, **kwargs):
        context = super(TriggerTypeEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context
    
    def form_valid(self, form):
        incident_type = IncidentType()
        incident_type.id = self.kwargs.get('incidentType')
        form.instance.incident_type = incident_type
        
        return super(TriggerTypeEdit, self).form_valid(form)

class TriggerTypeCreate(LoginRequiredMixin, CreateView):
    model = TriggerType
    template_name = "triggers/types/edit.html"
    form_class = TriggerTypeForm
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})
    
    def form_valid(self, form):
        incident_type = IncidentType()
        incident_type.id = self.kwargs.get('incidentType')
        form.instance.incident_type = incident_type
        
        return super(TriggerTypeCreate, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(TriggerTypeCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class TriggerTypeDelete(LoginRequiredMixin, DeleteView):
    model = TriggerType
    template_name = "triggers/types/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})

class CustomFieldCreate(LoginRequiredMixin, CreateView):
    model = CustomField
    form_class = CustomFieldForm
    template_name = "incidents/customField/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})
    
    def get_initial(self):
        initial = super(CustomFieldCreate, self).get_initial()
        
        initial['incident_type'] = self.kwargs.get('incidentType')
        
        return initial

class CustomFieldEdit(LoginRequiredMixin, UpdateView):
    model = CustomField
    form_class = CustomFieldForm
    template_name = "incidents/customField/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})
    
    def get_context_data(self, **kwargs):
        context = super(CustomFieldEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class CustomFieldDelete(LoginRequiredMixin, DeleteView):
    model = CustomField
    template_name = "incidents/customField/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-type-details', kwargs={'pk': self.kwargs.get('incidentType')})
#
# A provider subscription is an agreement between A and B, where A receives incident
# notifications from B. 
#
# The notification subscriptions are added by utilizing the API of B. 
#
## TODO: REWRITE TO USE EXTERNAL API
class ProviderSubscriptionCreate(LoginRequiredMixin, CreateView):
    model = NotificationType
    form_class = NotificationTypeForm
    template_name = "notifications/types/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('provider-details', kwargs={'pk': self.kwargs.get('provider')})
    
    def get_context_data(self, **kwargs):
        context = super(ProviderSubscriptionCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

#
# Subscriber Subscriptions are the opposite of Provider Subscriptions. Here B are able to add A
# This is handy in the case of e.g. local authorities - you would be able to add them yourself
# not having to wait for or ask them to add your system. 
#
class SubscriberSubscriptionCreate(LoginRequiredMixin, CreateView):
    model = NotificationType
    form_class = NotificationTypeForm
    template_name = "notifications/types/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-details', kwargs={'pk': self.kwargs.get('subscriber')})
    
    def form_valid(self, form):
        sub = Subscriber()
        sub.id = self.kwargs.get('subscriber')
        form.instance.subscriber = sub
        
        return super(SubscriberSubscriptionCreate, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberSubscriptionCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class SubscriberSubscriptionEdit(LoginRequiredMixin, UpdateView):
    model = NotificationType
    form_class = NotificationTypeForm
    template_name = "notifications/types/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-details', kwargs={'pk': self.kwargs.get('subscriber')})
    
    def form_valid(self, form):
        sub = Subscriber()
        sub.id = self.kwargs.get('subscriber')
        form.instance.subscriber = sub
        
        return super(SubscriberSubscriptionEdit, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberSubscriptionEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class SubscriberSubscriptionDetails(LoginRequiredMixin, DetailView):
    model = NotificationType
    template_name = "notifications/types/details.html"
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberSubscriptionDetails, self).get_context_data(**kwargs)
        context['incidents'] = NotificationIncident.objects.filter(notification_type = self.object)
        return context

class SubscriberSubscriptionDelete(LoginRequiredMixin, DeleteView):
    model = NotificationType
    template_name = "notifications/types/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'pk': self.kwargs.get('pk'), 'subscriber': self.kwargs.get('subscriber')})
    
    
class SubscriberSubscriptionIncidentCreate(LoginRequiredMixin, CreateView):
    model = NotificationIncident
    form_class = NotificationIncidentForm
    template_name = "notifications/incidents/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})
    
    def form_valid(self, form):
        type = NotificationType()
        type.id = self.kwargs.get('subscription')
        form.instance.notification_type = type
        
        return super(SubscriberSubscriptionIncidentCreate, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberSubscriptionIncidentCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class SubscriberSubscriptionIncidentEdit(LoginRequiredMixin, UpdateView):
    model = NotificationIncident
    form_class = NotificationIncidentForm
    template_name = "notifications/incidents/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})
    
    def form_valid(self, form):
        type = NotificationType()
        type.id = self.kwargs.get('subscription')
        form.instance.notification_type = type
        
        return super(SubscriberSubscriptionIncidentEdit, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriberSubscriptionIncidentEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class SubscriberSubscriptionIncidentDelete(LoginRequiredMixin, DeleteView):
    model = NotificationIncident
    template_name = "notifications/incidents/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})

class SubscriptionTriggerCreate(LoginRequiredMixin, CreateView):
    model = NotificationTrigger
    form_class = NotificationTriggerForm
    template_name = "notifications/triggers/edit.html"
    
    def get_form_kwargs(self):
        kwargs = super(SubscriptionTriggerCreate, self).get_form_kwargs()
        incidentId = self.kwargs.get('incident')
        id = uuid.UUID(incidentId)
        incident = NotificationIncident.objects.get(pk = id)
        kwargs['incidentType'] = incident.type
        
        return kwargs
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})
    
    def form_valid(self, form):
        incident = NotificationIncident()
        incident.id = self.kwargs.get('incident')
        form.instance.notification_incident = incident
        
        return super(SubscriptionTriggerCreate, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriptionTriggerCreate, self).get_context_data(**kwargs)
        context['edit'] = False
        return context

class SubscriptionTriggerEdit(LoginRequiredMixin, UpdateView):
    model = NotificationTrigger
    form_class = NotificationTriggerForm
    template_name = "notifications/triggers/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})
    
    def get_form_kwargs(self):
        kwargs = super(SubscriptionTriggerEdit, self).get_form_kwargs()
        incidentId = self.kwargs.get('incident')
        id = uuid.UUID(incidentId)
        incident = NotificationIncident.objects.get(pk = id)
        kwargs['incidentType'] = incident.type
        
        return kwargs
    
    def form_valid(self, form):
        incident = NotificationIncident()
        incident.id = self.kwargs.get('incident')
        form.instance.notification_incident = incident
        
        return super(SubscriptionTriggerEdit, self).form_valid(form)
    
    def get_context_data(self, **kwargs):
        context = super(SubscriptionTriggerEdit, self).get_context_data(**kwargs)
        context['edit'] = True
        return context

class SubscriptionTriggerDelete(LoginRequiredMixin, DeleteView):
    model = NotificationTrigger
    template_name = "notifications/triggers/delete.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('subscriber-subscription-details', kwargs={'subscriber': self.kwargs.get('subscriber'), 'pk': self.kwargs.get('subscription')})


class AttachmentCreate(LoginRequiredMixin, CreateView):
    model = Attachment
    form_class = AttachmentForm
    template_name = "incidents/attachments/edit.html"
    
    def get_success_url(self, **kwargs):
        return reverse_lazy('incident-details', kwargs={'pk': self.kwargs.get('incident')})
    
    def form_valid(self, form):
        incident = Incident()
        incident.id = self.kwargs.get('incident')
        form.instance.incident = incident
        
        return super(AttachmentCreate, self).form_valid(form)

#
# View for handling sending notifications to subscribers
#
class Notify(LoginRequiredMixin, View):
    def get(self, request, *args, **kwargs):
        incident_id = kwargs.get('incident')
        incident = Incident.objects.get(pk = incident_id)
         
        sender = Sender(request)
        sender.notify(incident)
        
        return redirect('incident-details', pk=kwargs.get('incident'), permanent=False)